aboutsummaryrefslogtreecommitdiffstats
path: root/Bootloaders/DFU
diff options
context:
space:
mode:
authorDean Camera <dean@fourwalledcubicle.com>2014-10-11 16:43:41 +1100
committerDean Camera <dean@fourwalledcubicle.com>2014-10-11 16:43:41 +1100
commit4c637c93ba69e33c77034eef9bd4433252e139fc (patch)
tree6d70fb9f07e93129aff9e6df8f9e45b10ca8cf58 /Bootloaders/DFU
parentfef991e28b56ce3e8de86833f00b6b7e77f1e84c (diff)
downloadlufa-4c637c93ba69e33c77034eef9bd4433252e139fc.tar.gz
lufa-4c637c93ba69e33c77034eef9bd4433252e139fc.tar.bz2
lufa-4c637c93ba69e33c77034eef9bd4433252e139fc.zip
Add Leonardo compatibility to most bootloaders, run user application if BOOTRST fuse is set and the last reset cause was not the /RESET pin.
Diffstat (limited to 'Bootloaders/DFU')
-rw-r--r--Bootloaders/DFU/BootloaderDFU.c35
-rw-r--r--Bootloaders/DFU/BootloaderDFU.txt14
2 files changed, 36 insertions, 13 deletions
diff --git a/Bootloaders/DFU/BootloaderDFU.txt b/Bootloaders/DFU/BootloaderDFU.txt
index e63bcc1f0..d0bb6e120 100644
--- a/Bootloaders/DFU/BootloaderDFU.txt
+++ b/Bootloaders/DFU/BootloaderDFU.txt
@@ -57,9 +57,17 @@
*
* \section Sec_Running Running the Bootloader
*
- * This bootloader is designed to be started via the HWB mechanism of the USB AVRs; ground the HWB pin (see device
- * datasheet) then momentarily ground /RESET to start the bootloader. This assumes the HWBE fuse is set and the BOOTRST
- * fuse is cleared.
+ * On the USB AVR8 devices, setting the \c HWBE device fuse will cause the bootloader to run if the \c HWB pin of
+ * the AVR is grounded when the device is reset.
+ *
+ * The are two behaviours of this bootloader, depending on the device's fuses:
+ *
+ * <b>If the device's BOOTRST fuse is set</b>, the bootloader will run any time the system is not reset from
+ * the external reset pin, unless no valid user application has been loaded. To initiate the bootloader, the
+ * device's external reset pin should be grounded.
+ *
+ * <b>If the device's BOOTRST fuse is not set</b>, the bootloader will run only if initiated via a software
+ * jump, or if the \c HWB pin was low during the last device reset (if the \c HWBE fuse is set).
*
* For board specific exceptions to the above, see below.
*
diff --git a/Bootloaders/DFU/BootloaderDFU.c b/Bootloaders/DFU/BootloaderDFU.c
index 160b9f657..cebc0f433 100644
--- a/Bootloaders/DFU/BootloaderDFU.c
+++ b/Bootloaders/DFU/BootloaderDFU.c
@@ -108,7 +108,17 @@ void Application_Jump_Check(void)
{
bool JumpToApplication = false;
- #if ((BOARD == BOARD_XPLAIN) || (BOARD == BOARD_XPLAIN_REV1))
+ #if (BOARD == BOARD_LEONARDO)
+ /* Enable pull-up on the IO13 pin so we can use it to select the mode */
+ PORTC |= (1 << 7);
+ Delay_MS(10);
+
+ /* If IO13 is not jumpered to ground, start the user application instead */
+ JumpToApplication = ((PINC & (1 << 7)) != 0);
+
+ /* Disable pull-up after the check has completed */
+ PORTC &= ~(1 << 7);
+ #elif ((BOARD == BOARD_XPLAIN) || (BOARD == BOARD_XPLAIN_REV1))
/* Disable JTAG debugging */
JTAG_DISABLE();
@@ -117,25 +127,30 @@ void Application_Jump_Check(void)
Delay_MS(10);
/* If the TCK pin is not jumpered to ground, start the user application instead */
- JumpToApplication |= ((PINF & (1 << 4)) != 0);
+ JumpToApplication = ((PINF & (1 << 4)) != 0);
/* Re-enable JTAG debugging */
JTAG_ENABLE();
- #endif
+ #else
+ /* Check if the device's BOOTRST fuse is set */
+ if (boot_lock_fuse_bits_get(GET_HIGH_FUSE_BITS) & FUSE_BOOTRST)
+ {
+ /* If the reset source was not an external reset or the key is correct, clear it and jump to the application */
+ if (!(MCUSR & (1 << EXTRF)) || (MagicBootKey == MAGIC_BOOT_KEY))
+ JumpToApplication = true;
- /* If the reset source was the bootloader and the key is correct, clear it and jump to the application */
- if ((MCUSR & (1 << WDRF)) && (MagicBootKey == MAGIC_BOOT_KEY))
- JumpToApplication |= true;
+ MCUSR &= ~(1 << EXTRF);
+ }
+ #endif
/* Don't run the user application if the reset vector is blank (no app loaded) */
-
- JumpToApplication = false;
+ bool ApplicationValid = (pgm_read_word_near(0) != 0xFFFF);
/* If a request has been made to jump to the user application, honor it */
- if (JumpToApplication)
+ if (JumpToApplication && ApplicationValid)
{
/* Turn off the watchdog */
- MCUSR &= ~(1<<WDRF);
+ MCUSR &= ~(1 << WDRF);
wdt_disable();
/* Clear the boot key and jump to the user application */